#!/usr/bin/perl -w

######################################################
#
#	Example of wrapper exploration
#
#	This script explore server performance vs. chip price
#	and produce a normalized TCO for each configuration 
#	in a file $filename.txt used after as an input of gnuplot
#
######################################################

use strict;

use POSIX qw/ceil/;

######################################################
#
# $basedir VARIABLE TO MODIFY if you change the 
# organization of directories
#
# put the basedir of the tco enterprise tool in $basedir
# example: my $basedir="/home/user/eetco/";
#
######################################################

my $basedir="../../";

######################################################
#
# Parameters to modify to create a new analysis
# assuming only one parameter have to be modified 
# for each axis.
#
# The first configuration  @xs[0];@ys[0] is used as 
# the reference for the TCO normalization
#
# You may need to change the computation of:
# 	my $x_value=ceil($x_ref/$x);	at the beginning of the x exploration loop
#	my $y_value= $y_ref * $y;		at the beginning of the y exploration loop
#
# and also the names of parameters in template/dcex_template.params
#
######################################################

#output file's name  without extension
my $filename="perf_vs_price";

#legende x y z for the gnuplot input file (starting with # and finished with\n)
my $legende= "#perf\t\tprice\t\tnormalizedTCO\n";

# configuration to explore: x and y

#nbserver_ref
my $x_ref = 50000;
my @xs=("1","1.1","1.2","1.3","1.4","1.5","1.6","1.7","1.8","1.9","2");#normalized exploration: $x_value=ceil($x_ref/$x)
my $x_template_name="XXSERVERS"; #name of the parameter in the template file

#price_ref
my $y_ref = 100;
my @ys=("1","1.1","1.2","1.3","1.4","1.5","1.6","1.7","1.8","1.9","2");#normalized exploration: $y_value= $y_ref * $y
my $y_template_name="XXPRICE"; #name of the parameter in the template file

####################################################
####################################################
####################################################

####################################################
# initialisation phase
####################################################

#to get the current dir
my $currentdir=`pwd`;
chomp $currentdir;

#open outputfile and print the legende
open(OUTPUTFILE,">$currentdir/output/$filename.txt") || die ("Error when opening $currentdir/output/$filename.txt") ;
print(OUTPUTFILE "$legende");

#goto the main directory to call the enterprise tool script
if($basedir eq "")
{
	print("Please, set the \$basedir variable before using this script\n");
	exit(0);
}
chdir($basedir) or die "$!";

my $first=1;
my $tco_ref=0;


####################################################
# main exploration loop
####################################################

#exploration loop of x (performance in that case)
foreach my $x (@xs)
{
	my $x_value=ceil($x_ref/$x);
	
	#exploration loop of y (price in that case)
	foreach my $y (@ys)
	{
		my $y_value= $y_ref * $y;
	
		#clean the previous configuration file
		if (-e "$currentdir/r.params") 
		{
			system("rm $currentdir/r.params");
		}
		if (-e "$currentdir/dc.params") 
		{
			system("rm $currentdir/dc.params");
		}
		
		#generate the current configuration file
		system("cat $currentdir/template/r_template.params | sed -e 's/$x_template_name/$x_value/g' | sed -e 's/$y_template_name/$y_value/g'  > $currentdir/r.params");
		system("cp  $currentdir/template/dc_template.params $currentdir/dc.params");

		#call the enterprise tool and keep the output
		system("./eetco.pl $currentdir/dc.params $currentdir/r.params  > $currentdir/output/x_$x_value._y_$y_value");
	
		#get the TCO value
		my $tco=`cat $currentdir/output/x_$x_value._y_$y_value | grep tco | gawk '{print \$2}'`;
		
		if($first==1) #if it is the first iteration, keep the TCO to normalized the TCO values
		{
			$first=0;
			$tco_ref=$tco;
		}
		
		my $normalized_tco=$tco/$tco_ref;
		
		print(OUTPUTFILE "$x \t $y \t $normalized_tco \n");
	}
	print(OUTPUTFILE "\n");#for gnuplot 3D plot
}

close(OUTPUTFILE);

#keep the output file in the current directory
system("cp $currentdir/output/$filename.txt $currentdir/$filename.txt");

#go back to the current dir
chdir($currentdir) or die "$!";

####################################################
# gnulpot call
####################################################
if (-e "./plot.plot") 
{
	system("rm ./plot.plot");
}	

system("cat ./template/gnuplot_template.plot | sed -e 's/XXNAME/$filename/g' > ./plot.plot");

system("gnuplot <plot.plot");


####################################################
# clean
####################################################
system("rm ./plot.plot");
system("rm ./dc.params");
system("rm ./r.params");
system("rm ./output/*");
#system("rm $currentdir/$filename.txt");
